# cython: language_level=3

cdef extern from "Zoltan_config.h":

    # package string
    #cdef str PACKAGE_STRING

    # package version
    #cdef str PACKAGE_VERSION

    # ZOLTAN_ID_TYPE
    cdef bint UNSIGNED_INT_GLOBAL_IDS
